// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: Statistic
// Author: Hassan Siddiqui
//
// Description: Computes simple statistics on-the-fly from a given input data list.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: Statistic.h,v 1.10 2000/06/30 11:49:51 hsiddiqu Exp $


#ifndef STATISTIC_H
#define STATISTIC_H

// General classes
#include <string> // put this up at the top as order matters
#include <iostream.h>
#include <math.h>
#include <list>

// SAS-specific classes
//#include "DataFilter.h"
#include "RowRange.h"
#include "StatisticElement.h"

// Utility functions
#include "errstr.h"
#include "Tracer.h"
#include "GenericOperation.h"

// ----------------------------------------------------------------

// class Statistic

// Computes simple statistics on-the-fly from a given input data list.

// NOTE : Before using any of the accessors, any code using this class must check whether
// an instance of this has any statistical information by checking the size() method.

// -----------------------------------------------------------------

class Statistic
{

public:

  /* Constructor */
  Statistic();

  Statistic(const Table& table, 
	    const string& name, 
	    const vector<RowRange>& rowranges);

  Statistic(const Column * col);

  void updateBasicStatistics(const StatisticElement& element);

  void updateDeviation(const StatisticElement& element);


  /* Accessors */
  const StatisticElement& max()  const;
  const StatisticElement& min() const;
  const double mean() const;
  const double sum() const;
  const double rootMeanSquared()   const;
  const double standardDeviation() const;
  const unsigned long size() const;


  /* Output stream method */

  void write(ostream& stream) const;

private:
  
  string _name;

  unsigned int _size;

  double _sum;

  double _sumOfSquares;

  double _deviation;

  StatisticElement _min;

  StatisticElement _max;

  bool _meanset; // Used to ensure that there is a mean value computed for the deviation calculation. This is a bit messy.

};


/* I/O Stream functions */

ostream &operator<<(ostream& stream, const Statistic& statistic);


#endif

